<?php

namespace ByDedi\Module\CustomForm\Back;

use BackLabel;
use BackModule;
use Query;
use Boutique;
use BackForm;

// phpcs:disable
class bForm_response extends BackModule
{
    const FIELD_NAME_PREFIX = "__field_";

    public function __construct($formvars = array())
    {
        parent::__construct($formvars);
        $this->docs = [
            "Global" => __DIR__ . "/doc/responses/index.md",
        ];
        $this->droits['ADD'] = false;
    }

    /**
     * @param array $formvars
     * @param null $table
     * @param bool $langue
     *
     * @SuppressWarnings("unused")
     */
    public function update($formvars = array(), $table = null, $langue = true)
    {
        foreach ($formvars as $name => $formvar) {
            if (preg_match("/" . self::FIELD_NAME_PREFIX . "([0-9]+)/", $name, $match)) {
                $this->sql->query(
                    "UPDATE form_field_response SET form_field_response_value = '$formvar'"
                    . "WHERE form_field_response_id = {$match[1]} "
                );
            }
        }
    }

    public function Form($formvars = array())
    {
        //-- Champs du formulaire
        $formvars += [
            'id' => 0,
            'div' => ''
        ];

        $response = $this->sql->getOne(getTable('form_response', $formvars['id']));

        $request = "SELECT 1";

        $formId = $response['form_id'];
        $responseId = $formvars['id'];

        $fields = $this->getFields($formId);
        array_walk($fields, function (array $field) use ($responseId, &$request) {
            $fieldId = $field['form_field_id'];
            $fieldResponse = $this->getFieldResponse($fieldId, $responseId);
            $fieldName = self::FIELD_NAME_PREFIX . "{$fieldResponse['form_field_response_id']}";
            $isMessageField = strtolower($field['form_field_nom_nom']) === "message";
            $responseText = addslashes($fieldResponse['form_field_response_value']);
            $form = new BackForm($field['form_field_nom_nom'], ($isMessageField ? "textarea" : "text"), $fieldName);
            $request .= ", '{$responseText}' AS `$fieldName`";
            $this->addForm($form);
        });

        $this->request = $request;

        return $this->displayForm($formvars['id'], $formvars['div']);
    }

    public function Listing($formvars = array())
    {
        $query = new Query();
        $query
            ->select(
                'f.form_id',
                'fn.form_nom_nom'
            )
            ->from('form')
            ->leftJoin('form_nom', "AND", ['fn.form_id', 'f.form_id'], ['fn.langue_id', Boutique::getInstance()->getLangue()->id])
        ;

        $form = new BackForm("Form", "select", "fr.form_id");
        $form->addOption("", "---");
        $form->addOptionSQL([$query->build()]);
        $form->setVar('compare', "SPEC");
        $this->addForm($form);

        $form = new BackForm("Date", "between", "fr.form_response_date_ajout");
        $this->addForm($form);

        if (!empty($formvars['form'])) {
            $formId = $formvars['form'];
        }

        if (!empty($_GET['fr*form_id'])) {
            $formId = $_GET['fr*form_id'];
        }

        $query = new Query();
        $query
            ->select('fr.form_response_id')
            ->select('fr.form_response_id')
            ->select('fr.form_response_date_ajout')
            ->from('form_response')
        ;

        $label = new BackLabel("ID");
        $this->addLabel($label);

        $label = new BackLabel("DATE");
        $this->addLabel($label);

        if (!empty($formId)) {
            $res = $this->getFields($formId);
            $query->where('fr.form_id', $formId);

            array_walk($res, function (array $fieldResponse, $fieldResponseIndex) use ($query) {
                $query->select("ffr$fieldResponseIndex.form_field_response_value")
                    ->leftJoin(
                        "form_field_response ffr$fieldResponseIndex",
                        "AND",
                        ["ffr$fieldResponseIndex.form_response_id", "fr.form_response_id"],
                        ["ffr$fieldResponseIndex.form_field_id", $fieldResponse['form_field_id']]
                    )
                ;

                $label = new BackLabel($fieldResponse['form_field_nom_nom']);
                $this->addLabel($label);
            });
        } else {
            $query
                ->select("CONCAT(fn.form_id, '_', fn.form_nom_nom)")
                ->innerJoin('form_nom', 'AND', ['fn.form_id', 'fr.form_id'], ['fn.langue_id', Boutique::getInstance()->getLangue()->id])
            ;

            $label = new BackLabel("FORMULAIRE", 'listselect');
            $label->setVar('option', array("module" => "<a href='../../form/form/%1%/'>%2%</a>"));
            $this->addLabel($label);
        }

        $this->request = $query->build();
        //-- SQL
        // LABELS

        return $this->displayList($formvars['type']);
    }

    /**
     * @param $formId
     * @return array
     */
    private function getFields($formId)
    {
        $query = new Query();
        $query
            ->select('*')
            ->from('form_field')
            ->leftJoin(
                'form_field_nom',
                "AND",
                ['ffn.form_field_id', 'ff.form_field_id'],
                ['ffn.langue_id', Boutique::getInstance()->getLangue()->id]
            )
            ->where('form_id', $formId)
        ;
        $res = $this->sql->getAll($query->build());
        return $res;
    }

    /**
     * @param $fieldId
     * @param $responseId
     * @return array
     */
    private function getFieldResponse($fieldId, $responseId)
    {
        $query = new Query();
        $query
            ->select()
            ->from('form_field_response')
            ->where('ffr.form_field_id', $fieldId)
            ->where('ffr.form_response_id', $responseId)
        ;
        $fieldResponse = $this->sql->getOne($query->build());
        return $fieldResponse;
    }

//--
}
// phpcs:enable
