<?php
/**
 * Created by PhpStorm.
 * User: gabriel
 * Date: 10/09/18
 * Time: 13:41
 */

namespace ByDedi\Module\CustomForm\Model;

use Boutique;
use Form\Constraint\ConstraintInterface;
use Form\Constraint\Mail;
use Form\Constraint\NotBlank;
use Form\Constraint\Phone;
use LogicException;

/**
 * Class Constraints
 * @package ByDedi\Module\CustomForm\Model
 *
 * @property int form_constraint_id
 * @property string form_constraint_libelle
 * @property string form_field_constraint_nom_error
 */
class Constraint extends Model implements CustomConstraintInterface
{
    const CONSTRAINT_NOT_BLANK_ID = 1;
    const CONSTRAINT_EMAIL_ID = 2;
    const CONSTRAINT_FRENCH_PHONE_ID = 3;

    /**
     * @var string
     */
    protected $table = "form_field_constraint";

    /**
     * @return int
     */
    public function getId()
    {
        return $this->form_constraint_id;
    }

    /**
     * @return string
     */
    public function getFormConstraintLibelle()
    {
        return $this->form_constraint_libelle;
    }

    public function getQuery($formvars = [])
    {
        $query = parent::getQuery($formvars);
        $boutique = Boutique::getInstance();
        $query
            ->innerJoin(
                'form_field_constraint_boutique',
                'AND',
                ['ffcb.form_field_constraint_id', 'ffc.form_field_constraint_id'],
                ['ffcb.boutique_id', $boutique->id],
                ['ffcb.form_field_constraint_boutique_statut_id', 1]
            )
        ;

        return $query;
    }

    /**
     * @return ConstraintInterface
     */
    private function getConstraint()
    {
        $constraintId = $this->getId();
        $constraints = [
            self::CONSTRAINT_NOT_BLANK_ID => new NotBlank(),
            self::CONSTRAINT_EMAIL_ID => new Mail(),
            self::CONSTRAINT_FRENCH_PHONE_ID => new Phone(),
        ];

        if (!isset($constraints[$constraintId])) {
            throw new LogicException("No constraint matches id '$constraintId'");
        }

        return $constraints[$constraintId];
    }

    /**
     * @param $value
     *
     * @return bool
     */
    public function isValid($value)
    {
        return $this->getConstraint()->isValid($value);
    }

    /**
     * @return string
     */
    public function getError()
    {
        return $this->form_field_constraint_nom_error;
    }

    /**
     * @param FieldEntityInterface $fieldEntity
     * @return static[]
     */
    public function getByField(FieldEntityInterface $fieldEntity)
    {
        if (empty($fieldEntity->getId())) {
            trigger_error("Not loaded FormFieldEntity provided in " . __METHOD__);
            return [];
        }
        return $this->getLoaded(['ffc.form_field_id' => $fieldEntity->getId()]);
    }

    /**
     * @return bool
     */
    public function isMail()
    {
        return $this->getConstraint() instanceof Mail;
    }

    /**
     * @return bool
     */
    public function isPhone()
    {
        return $this->getConstraint() instanceof Phone;
    }
}
