<?php

namespace ByDedi\Module\CustomForm\Model;

use Boutique;
use Form\Constraint\ConstraintInterface;
use Form\FullForm\FieldTrait;

/**
 * Class FormField
 *
 * @property int $form_field_id
 * @property int $form_field_type_id
 * @property string form_field_nom_nom
 * @property string form_field_tag
 */
class FieldEntity extends Model implements FieldEntityInterface
{
    const STATUT_ACTIVE = 1;
    use FieldTrait;

    /**
     * @var string
     */
    protected $table = "form_field";

    /**
     * @var Option[]|null
     */
    private $options;

    /**
     * @var Constraint[]|null
     */
    private $constraints;

    /**
     * @return int
     */
    public function getId()
    {
        return $this->form_field_id;
    }

    /**
     * @return int
     */
    public function getTypeId()
    {
        return $this->form_field_type_id;
    }

    /**
     * @return mixed
     */
    protected function getValue()
    {
        return '';
    }

    /**
     * @return string
     */
    public function getLabel()
    {
        return $this->form_field_nom_nom;
    }

    /**
     * @return Constraint[]
     */
    public function getConstraints()
    {
        if ($this->constraints === null) {
            if (empty($this->getId())) {
                return [];
            }

            $constraint = new Constraint();
            $this->constraints = $constraint->getByField($this);
        }

        return $this->constraints;
    }

    /**
     * {@inheritdoc}
     * @return Option[]
     */
    public function getOptions()
    {
        if ($this->options === null) {
            if (empty($this->getId())) {
                return [];
            }

            $option = new OptionRepository();
            $this->options = $option->getByField($this);
        }

        return $this->options;
    }

    /**
     * @param FormEntityInterface $formEntity
     * @return static[]
     */
    public function getByForm(FormEntityInterface $formEntity)
    {
        return $this->getLoaded(['ff.form_id' => $formEntity->getId()]);
    }

    /**
     * @param array $formvars
     * @return mixed
     */
    public function getQuery($formvars = [])
    {
        $boutique = Boutique::getInstance();
        $query = parent::getQuery($formvars);
        $query
            ->innerJoin(
                'form_field_boutique',
                'AND',
                ['ffb.form_field_id', 'ff.form_field_id'],
                ['ffb.boutique_id', $boutique->id],
                ['ffb.form_field_boutique_statut_id', 1]
            )
            ->where('form_field_statut_id', self::STATUT_ACTIVE)
            ->order('ff.form_field_rang')
        ;

        return $query;
    }

    /**
     * @return array
     */
    public function getTags()
    {
        return array_filter(explode(' ', $this->form_field_tag));
    }
}
