<?php
/**
 * Created by PhpStorm.
 * User: gabriel
 * Date: 25/05/18
 * Time: 12:46
 */

namespace ByDedi\Module\CustomForm\Model;

use ByDedi\Module\CustomForm\Service\NoIdException;
use InvalidArgumentException;
use LogicException;

/**
 * Class FormResponseField
 *
 * @property int form_field_response_id
 * @property int $form_field_id
 * @property int $form_response_id
 * @property string $form_field_response_value
 * @property int $form_field_option_id
 */
class FieldResponse extends Model implements FieldResponseInterface
{
    /**
     * @var string
     */
    protected $table = 'form_field_response';

    /**
     * @var FormResponse
     */
    private $response;

    /**
     * @var FieldEntityInterface
     */
    private $field;

    /**
     * @var OptionEntityInterface|null
     */
    private $option;

    /**
     * FormResponseField constructor.
     * @param FormResponse $formResponse
     * @param FieldEntityInterface $fieldEntity
     * @param $value
     * @param OptionEntityInterface|null $option
     */
    public function __construct(
        FormResponse $formResponse,
        FieldEntityInterface $fieldEntity,
        $value,
        OptionEntityInterface $option = null
    ) {
        $fieldid = $fieldEntity->getId();
        if (empty($fieldid)) {
            throw new InvalidArgumentException("Field must have an id to be saved as a response");
        }

        parent::__construct();
        $this->response = $formResponse;
        $this->field = $fieldEntity;
        $this->form_field_response_value = $value;
        $this->option = $option;
    }

    /**
     * {@inheritdoc}
     * @throws LogicException
     */
    public function save($data = null)
    {
        $formResponse = $this->getResponse();
        $formFieldId = $this->getFormField()->getId();
        $option = $this->getOption();

        if (empty($formFieldId)) {
            throw new LogicException("FormField must have an id to be saved as a response");
        }
        if (!$formResponse->hasId()) {
            throw new LogicException("FormResponse must have an id to save a FormResponseField");
        }

        $this->form_response_id = $formResponse->getId();
        $this->form_field_id = $formFieldId;
        $this->form_field_option_id = $option !== null ? $option->getId() : null;

        $responseFieldId = parent::save($data);
        $this->form_field_response_id = $responseFieldId;

        return $this->getId();
    }

    /**
     * {@inheritdoc}
     */
    public function hasId()
    {
        return $this->form_field_response_id !== null;
    }

    /**
     * {@inheritdoc}
     */
    public function getId()
    {
        if (!$this->hasId()) {
            throw new NoIdException();
        }
        return $this->form_field_response_id;
    }

    /**
     * @return FormResponse
     */
    public function getResponse()
    {
        return $this->response;
    }

    /**
     * @return FieldEntityInterface
     */
    public function getFormField()
    {
        return $this->field;
    }

    /**
     * @return string
     */
    public function getValue()
    {
        return $this->form_field_response_value;
    }

    /**
     * @return OptionEntityInterface|null
     */
    public function getOption()
    {
        return $this->option;
    }
}
