<?php

namespace ByDedi\Module\CustomForm\Model;

use Boutique;

/**
 * Class CustomForm
 *
 * @property int $form_id
 * @property string $form_nom_success
 * @property string form_nom_submit_text
 */
class FormEntity extends Model implements FormEntityInterface
{
    const FORM_STATUT_ACTIVE = 1;

    /**
     * @var string
     */
    protected $table = "form";

    /**
     * @var FieldEntity[]
     */
    protected $fields;

    public function getQuery($formvars = [])
    {
        $query = parent::getQuery($formvars);
        $boutique = Boutique::getInstance();
        $query
            ->innerJoin(
                'form_boutique',
                'AND',
                ['fb.form_id', 'f.form_id'],
                ['fb.boutique_id', $boutique->id],
                ['fb.form_boutique_statut_id', 1]
            )
            ->where('f.form_statut_id', self::FORM_STATUT_ACTIVE)
        ;

        return $query;
    }

    /**
     * @return FieldEntity[]
     */
    public function getFields()
    {
        if ($this->fields === null) {
            if (empty($this->getId())) {
                return [];
            }

            $field = new FieldEntity();
            $this->fields = $field->getByForm($this);
        }

        return $this->fields;
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->form_id;
    }

    /**
     * {@inheritdoc}
     */
    public function getSuccessText()
    {
        return $this->form_nom_success;
    }

    /**
     * {@inheritdoc}
     */
    public function getSubmitButtonText()
    {
        return $this->form_nom_submit_text;
    }
}
