<?php
/**
 * Created by PhpStorm.
 * User: gabriel
 * Date: 10/09/18
 * Time: 13:10
 */

namespace ByDedi\Module\CustomForm\Model;

use Form\FullForm\OptionInterface;

/**
 * Class Option
 * @package ByDedi\Module\CustomForm\Model
 *
 * @property int form_field_option_id
 * @property int form_field_id
 * @property string form_field_option_value
 * @property string form_field_option_nom_nom
 */
class Option extends Model implements OptionEntityInterface
{
    /**
     * @var string
     */
    protected $table = "form_field_option";

    /**
     * @var bool
     */
    private $selected;

    /**
     * Option constructor.
     * @param int $optionId
     * @param string $value
     * @param string $name
     */
    public function __construct(
        $optionId,
        $value,
        $name,
        $selected = false
    ) {
        parent::__construct();
        $this->form_field_option_id = $optionId;
        $this->form_field_option_value = $value;
        $this->form_field_option_nom_nom = $name;
        $this->selected = $selected;
    }


    /**
     * @return int
     */
    public function getId()
    {
        return $this->form_field_option_id;
    }

    /**
     * @return string
     */
    public function getValue()
    {
        return $this->form_field_option_value;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->form_field_option_nom_nom;
    }

    /**
     * @param string $value
     * @return OptionInterface
     */
    public function withValue($value)
    {
        return new self(
            $this->getId(),
            $value,
            $this->getName(),
            $this->isSelected()
        );
    }

    /**
     * @param string $name
     * @return OptionInterface
     */
    public function withName($name)
    {
        return new self(
            $this->getId(),
            $this->getValue(),
            $name,
            $this->isSelected()
        );
    }

    /**
     * @return bool
     */
    public function isSelected()
    {
        return $this->selected;
    }

    /**
     * @param bool $selected
     * @return OptionInterface
     */
    public function withSelected($selected)
    {
        return new self(
            $this->getId(),
            $this->getValue(),
            $this->getName(),
            $selected
        );
    }

    /**
     * @return bool
     */
    public function isDisabled()
    {
        return false;
    }

    /**
     * @param bool $disabled
     * @return OptionInterface
     *
     * @SuppressWarnings("unused")
     */
    public function withDisabled($disabled)
    {
        return $this;
    }
}
