<?php
// phpcs:disable

use Phinx\Migration\AbstractMigration;

class CustomForm extends AbstractMigration
{
    /**
     * Change Method.
     *
     * Write your reversible migrations using this method.
     *
     * More information on writing migrations is available here:
     * http://docs.phinx.org/en/latest/migrations.html#the-abstractmigration-class
     *
     * The following commands can be used in this method and Phinx will
     * automatically reverse them when rolling back:
     *
     *    createTable
     *    renameTable
     *    addColumn
     *    renameColumn
     *    addIndex
     *    addForeignKey
     *
     * Remember to call "create()" or "update()" and NOT "save()" when working
     * with the Table class.
     *
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    public function change()
    {
        $isLangueIdSigned = $this->isTablePkSigned('langue');
        $isBoutiqueIdSigned = $this->isTablePkSigned('boutique');

        $this->table('form', ['id' => 'form_id'])
            ->addColumn('form_statut_id', 'integer', ['default' => 1])
            ->create()
        ;

        $this->table('form_nom', ['id' => 'form_nom_id'])
            ->addColumn('form_nom_nom', 'string')
            ->addColumn('form_nom_success', 'text')
            ->addColumn('form_id', 'integer')
            ->addColumn('langue_id', 'integer', ['signed' => $isLangueIdSigned])
            ->addForeignKey(
                'form_id',
                'form',
                'form_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addForeignKey(
                'langue_id',
                'langue',
                'langue_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addIndex(['form_id', 'langue_id'], ['unique' => true])
            ->create()
        ;

        $this->table('form_boutique', ['id' => 'form_boutique_id'])
            ->addColumn('boutique_id', 'integer', ['signed' => $isBoutiqueIdSigned])
            ->addColumn('form_id', 'integer')
            ->addForeignKey(
                'boutique_id',
                'boutique',
                'boutique_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addForeignKey(
                'form_id',
                'form',
                'form_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addIndex(['form_id', 'boutique_id'], ['unique' => true])
            ->create()
        ;

        $this->table('form_constraint', ['id' => 'form_constraint_id'])
            ->addColumn('form_constraint_libelle', 'string')
            ->create()
        ;

        $this->table('form_constraint')
            ->insert([
                [
                    'form_constraint_id' => 1,
                    'form_constraint_libelle' => 'Obligatoire'
                ],
                [
                    'form_constraint_id' => 2,
                    'form_constraint_libelle' => 'Email'
                ],
                [
                    'form_constraint_id' =>  3,
                    'form_constraint_libelle' => 'Numéro de téléphone (Français)'
                ],

            ])
            ->saveData()
        ;

        $this->table('form_field', ['id' => 'form_field_id'])
            ->addColumn('form_field_statut_id', 'integer', ['default' => 1])
            ->addColumn('form_id', 'integer')
            ->addForeignKey(
                'form_id',
                'form',
                'form_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->create()
        ;

        $this->table('form_field_nom', ['id' => 'form_field_nom_id'])
            ->addColumn('form_field_nom_nom', 'string')
            ->addColumn('form_field_id', 'integer')
            ->addColumn('langue_id', 'integer', ['signed' => $isLangueIdSigned])
            ->addForeignKey(
                'form_field_id',
                'form_field',
                'form_field_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addForeignKey(
                'langue_id',
                'langue',
                'langue_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addIndex(['form_field_id', 'langue_id'], ['unique' => true])
            ->create()
        ;

        $this->table('form_field_boutique', ['id' => 'form_field_boutique_id'])
            ->addColumn('boutique_id', 'integer', ['signed' => $isBoutiqueIdSigned])
            ->addColumn('form_field_id', 'integer')
            ->addForeignKey(
                'boutique_id',
                'boutique',
                'boutique_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addForeignKey(
                'form_field_id',
                'form_field',
                'form_field_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addIndex(['form_field_id', 'boutique_id'], ['unique' => true])
            ->create()
        ;

        $this->table('form_field_constraint', ['id' => 'form_field_constraint_id'])
            ->addColumn('form_constraint_id', 'integer')
            ->addColumn('form_field_id', 'integer')
            ->addForeignKey(
                'form_constraint_id',
                'form_constraint',
                'form_constraint_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addForeignKey(
                'form_field_id',
                'form_field',
                'form_field_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addIndex(['form_field_id', 'form_constraint_id'], ['unique' => true])
            ->create()
        ;

        $this->table('form_field_constraint_nom', ['id' => false, 'primary_key' => ['form_field_constraint_nom_id']])
            ->addColumn('form_field_constraint_nom_id', 'integer', ['signed' => false, 'identity' => 'enabled'])
            ->addColumn('form_field_constraint_id', 'integer')
            ->addColumn('langue_id', 'integer', ['signed' => $isLangueIdSigned])
            ->addColumn('form_field_constraint_nom_error', 'string')
            ->addIndex(['form_field_constraint_id', 'langue_id'], ['unique' => true])
            ->addForeignKey(
                'form_field_constraint_id',
                'form_field_constraint',
                'form_field_constraint_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addForeignKey(
                'langue_id',
                'langue',
                'langue_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->create()
        ;
    }

    /**
     * @param $tableName
     * @return bool
     */
    private function isTablePkSigned($tableName)
    {
        $tableDesc = $this->query("DESC $tableName");
        foreach ($tableDesc as $field) {
            $normalizedField = array_reduce(array_keys($field), function (array $res, $key) use ($field) {
                $res[strtolower($key)] = $field[$key];
                return $res;
            }, []);
            if ($normalizedField['field'] === $tableName . '_id') {
                $isLangueIdSigned = !preg_match('/unsigned/', $normalizedField['type']);
                break;
            }
        }
        if (!isset($isLangueIdSigned)) {
            trigger_error("No {$tableName}_id found in table `$tableName`. Defaulted to signed");
            $isLangueIdSigned = false;
        }
        return $isLangueIdSigned;
    }
}
