<?php
// phpcs:disable

use Phinx\Migration\AbstractMigration;

class FormConstraintFieldBoutique extends AbstractMigration
{
    /**
     * Change Method.
     *
     * Write your reversible migrations using this method.
     *
     * More information on writing migrations is available here:
     * http://docs.phinx.org/en/latest/migrations.html#the-abstractmigration-class
     *
     * The following commands can be used in this method and Phinx will
     * automatically reverse them when rolling back:
     *
     *    createTable
     *    renameTable
     *    addColumn
     *    renameColumn
     *    addIndex
     *    addForeignKey
     *
     * Remember to call "create()" or "update()" and NOT "save()" when working
     * with the Table class.
     */
    public function change()
    {
        $isBoutiqueIdSigned = $this->isTablePkSigned('boutique');
        $this->table('form_field_constraint_boutique', ['id' => 'form_field_constraint_boutique_id'])
            ->addColumn('form_field_constraint_id', 'integer')
            ->addColumn('boutique_id', 'integer', ['signed' => $isBoutiqueIdSigned])
            ->addColumn('form_field_constraint_boutique_statut_id', 'integer', ['default' => 1])
            ->addForeignKey(
                'boutique_id',
                'boutique',
                'boutique_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addForeignKey(
                'form_field_constraint_id',
                'form_field_constraint',
                'form_field_constraint_id',
                ['delete' => 'CASCADE', 'update' => 'CASCADE']
            )
            ->addIndex(['form_field_constraint_id', 'boutique_id'], ['unique' => true])
            ->create()
        ;
    }

    /**
     * @param $tableName
     * @return bool
     */
    private function isTablePkSigned($tableName)
    {
        $tableDesc = $this->query("DESC $tableName");
        foreach ($tableDesc as $field) {
            $normalizedField = array_reduce(array_keys($field), function (array $res, $key) use ($field) {
                $res[strtolower($key)] = $field[$key];
                return $res;
            }, []);
            if ($normalizedField['field'] === $tableName . '_id') {
                $isLangueIdSigned = !preg_match('/unsigned/', $normalizedField['type']);
                break;
            }
        }
        if (!isset($isLangueIdSigned)) {
            trigger_error("No {$tableName}_id found in table `$tableName`. Defaulted to signed");
            $isLangueIdSigned = false;
        }
        return $isLangueIdSigned;
    }
}
