<?php
/**
 * Created by PhpStorm.
 * User: gabriel
 * Date: 24/05/18
 * Time: 13:10
 */

namespace ByDedi\Module\CustomForm\Service;

use ByDedi\Module\CustomForm\Model\CustomConstraintInterface;
use ByDedi\Module\CustomForm\Model\FieldEntityInterface;
use Form\FullForm\FieldTrait;

/**
 *
 */
class FieldFromEntity implements FieldFromEntityInterface
{
    use FieldTrait;

    /**
     * @var FieldEntityInterface
     */
    private $formField;

    /**
     * @var string
     */
    private $name;

    /**
     * RenderableField constructor.
     * @param $name
     * @param FieldEntityInterface $formField
     */
    public function __construct($name, FieldEntityInterface $formField)
    {
        $this->formField = $formField;
        $this->name = $name;
    }

    /**
     * {@inheritdoc}
     */
    public function isRequired()
    {
        return !$this->isValid();
    }

    /**
     * {@inheritdoc}
     */
    public function getOptions()
    {
        return $this->getFieldEntity()->getOptions();
    }

    /**
     * {@inheritdoc}
     */
    public function getConstraints()
    {
        return $this->getFieldEntity()->getConstraints();
    }

    /**
     * {@inheritdoc}
     */
    public function getTypeId()
    {
        return $this->getFieldEntity()->getTypeId();
    }

    /**
     * {@inheritdoc}
     */
    public function getId()
    {
        return $this->getFieldEntity()->getId();
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * {@inheritdoc}
     */
    public function getValue()
    {
        return '';
    }

    /**
     * @return string
     */
    public function getLabel()
    {
        return $this->getFieldEntity()->getLabel();
    }

    /**
     * Indicate that the field was incorrectly filled
     *
     * @return bool
     */
    public function hasError()
    {
        return false;
    }

    /**
     * @return string
     */
    public function getError()
    {
        return '';
    }

    /**
     * {@inheritdoc}
     */
    public function getInputType()
    {
        if ($this->isMail()) {
            return 'email';
        }

        if ($this->isPhone()) {
            return 'tel';
        }

        return 'text';
    }

    /**
     * @return string|null
     */
    public function getPattern()
    {
        return null;
    }

    /**
     * @return bool
     */
    public function isAutocompleteOff()
    {
        return false;
    }

    /**
     * Indicates whether there should be an autofocus attribute
     *
     * @return bool
     */
    public function isFocused()
    {
        return false;
    }

    /**
     * @return bool
     */
    public function isDisabled()
    {
        return false;
    }

    /**
     * @return bool
     */
    public function isReadOnly()
    {
        return false;
    }

    /**
     * @return string
     */
    public function getPlaceHolder()
    {
        return '';
    }

    /**
     * {@inheritdoc}
     */
    public function isDirty()
    {
        return false;
    }

    /**
     * @return FieldEntityInterface
     */
    public function getFieldEntity()
    {
        return $this->formField;
    }

    /**
     * @return bool
     */
    private function isMail()
    {
        return array_reduce(
            $this->getConstraints(),
            function ($hasConstraint, CustomConstraintInterface $constraint) {
                return $hasConstraint || $constraint->isMail();
            },
            false
        );
    }

    /**
     * @return bool
     */
    private function isPhone()
    {
        return array_reduce(
            $this->getConstraints(),
            function ($hasConstraint, CustomConstraintInterface $constraint) {
                return $hasConstraint || $constraint->isPhone();
            },
            false
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getTags()
    {
        return $this->getFieldEntity()->getTags();
    }
}
