<?php

namespace ByDedi\Module\CustomForm\Service;

use ByDedi\Module\CustomForm\Model\FormEntityInterface;
use Form\FullForm\FormTrait;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class FilledForm
 */
class FilledFormFromEntity implements FormFromEntityInterface
{
    use FormTrait;

    /**
     * @var FormFromEntityInterface
     */
    private $form;

    /**
     * @var Request
     */
    private $request;

    /**
     * FilledForm constructor.
     * @param FormFromEntityInterface $form
     * @param Request $request
     */
    public function __construct(FormFromEntityInterface $form, Request $request)
    {
        $this->form = $form;
        $this->request = $request;
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->getFormEntity()->getId();
    }

    /**
     * @return string
     */
    public function getSuccessText()
    {
        return $this->getForm()->getSuccessText();
    }

    /**
     * @return string
     */
    public function getErrorText()
    {
        return $this->getForm()->getErrorText();
    }

    /**
     * @return FilledFieldFromEntity[]
     */
    public function getFields()
    {
        return array_map(function (FieldFromEntityInterface $fieldAndEntity) {
            return new FilledFieldFromEntity($fieldAndEntity, $this->getRequest());
        }, $this->getForm()->getFields());
    }

    /**
     * returns a value that can be used as the name attribute of a html form
     *
     * @return string
     */
    public function getName()
    {
        return $this->getForm()->getName();
    }

    /**
     * @return string
     */
    public function getSubmitButtonText()
    {
        return $this->getForm()->getSubmitButtonText();
    }

    /**
     * @return bool
     */
    public function hasError()
    {
        return !$this->isValid() || $this->getForm()->getErrorText() !== null;
    }

    /**
     * {@inheritdoc}
     */
    public function getFormEntity()
    {
        return $this->getForm()->getFormEntity();
    }

    /**
     * @return FormFromEntityInterface
     */
    public function getForm()
    {
        return $this->form;
    }

    /**
     * @return Request
     */
    private function getRequest()
    {
        return $this->request;
    }

    /**
     * Returns an instance of FormInterface for which hasError() SHOULD return true
     * Returned instance getErrorText CAN return $text if provided
     *
     * @param string $text
     * @return static
     */
    public function withError($text = null)
    {
        return new self($this->getForm()->withError($text), $this->getRequest());
    }

    /**
     * Returns an instance of FormInterface for which hasError() SHOULD return false
     * Returned instance getErrorText SHOULD return an empty string
     *
     * @return static
     */
    public function withoutError()
    {
        return new self($this->getForm()->withoutError(), $this->getRequest());
    }
}
