<?php

namespace ByDedi\Module\CustomForm\Service;

use ByDedi\Module\CustomForm\Model\FieldEntityInterface;
use ByDedi\Module\CustomForm\Model\FormEntityInterface;
use Form\FullForm\FieldInterface;
use Form\FullForm\FormTrait;

/**
 *
 */
class FormFromEntity implements FormFromEntityInterface
{
    use FormTrait;

    /**
     * @var FormEntityInterface $formEntity
     */
    protected $formEntity;

    /**
     * @var string|null
     */
    private $errorText;

    /**
     * CustomForm constructor.
     * @param FormEntityInterface $form
     * @param null $errorText
     */
    public function __construct(FormEntityInterface $form, $errorText = null)
    {
        $this->formEntity = $form;
        $this->errorText = $errorText;
    }

    /**
     * @return FormEntityInterface
     */
    public function getFormEntity()
    {
        return $this->formEntity;
    }

    /**
     * {@inheritdoc}
     */
    public function getSuccessText()
    {
        return $this->getFormEntity()->getSuccessText();
    }

    /**
     * @return FieldFromEntityInterface[]
     */
    public function getFields()
    {
        return array_map(function (FieldEntityInterface $field) {
            return new FieldFromEntity(
                $this->getName() . "_field_{$field->getId()}",
                $field
            );
        }, $this->getFormEntity()->getFields());
    }

    /**
     * {@inheritdoc}
     */
    public function getId()
    {
        return $this->getFormEntity()->getId();
    }

    /**
     * @return string
     */
    public function getErrorText()
    {
        return $this->errorText;
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return "__custom_form_{$this->getId()}";
    }

    /**
     * @return string
     */
    public function getSubmitButtonText()
    {
        return $this->getFormEntity()->getSubmitButtonText();
    }

    /**
     * @return bool
     */
    public function hasError()
    {
        return $this->getErrorText() !== null;
    }

    /**
     * Returns an instance of FormInterface for which hasError() SHOULD return true
     * Returned instance getErrorText CAN return $text if provided
     *
     * @param string $text
     * @return static
     */
    public function withError($text = null)
    {
        return new self($this->getFormEntity(), $text);
    }

    /**
     * Returns an instance of FormInterface for which hasError() SHOULD return false
     * Returned instance getErrorText SHOULD return an empty string
     *
     * @return static
     */
    public function withoutError()
    {
        return new self($this->getFormEntity());
    }
}
