<?php
/**
 * Created by PhpStorm.
 * User: gabriel
 * Date: 24/05/18
 * Time: 13:59
 */

namespace ByDedi\Module\CustomForm\Service;

use ByDedi\Module\CustomForm\Model\FormResponse;
use ByDedi\Module\CustomForm\Model\FieldResponse;
use ByDedi\Module\CustomForm\Model\OptionEntityInterface;
use InvalidArgumentException;

class ResponseFactory
{
    /**
     * @param FormFromEntityInterface $form
     * @return FormResponse
     */
    public function generate(FormFromEntityInterface $form)
    {
        if (!$form->isValid()) {
            throw new InvalidArgumentException("Trying to save invalid form");
        }

        $response = new FormResponse($form->getFormEntity());
        $responseFields = $this->getFormFieldResponses($form, $response);
        $response->addFields(...$responseFields);

        return $response;
    }

    /**
     * @param FormFromEntityInterface $form
     * @param $response
     * @return FieldResponse[]
     */
    private function getFormFieldResponses(FormFromEntityInterface $form, $response)
    {
        $responseFields = array_map(function (FieldFromEntityInterface $fieldAndEntity) use ($response) {
            $value = $fieldAndEntity->getValue();
            $option = $fieldAndEntity->getOptionByValue($value);
            return new FieldResponse(
                $response,
                $fieldAndEntity->getFieldEntity(),
                $value,
                ($option instanceof OptionEntityInterface) ? $option : null
            );
        }, $form->getFields());
        return $responseFields;
    }
}
