<?php

namespace ByDedi\Module\CustomForm\Tests\Service;

use ByDedi\Module\CustomForm\Model\FormEntityInterface;
use ByDedi\Module\CustomForm\Model\FieldEntityInterface;
use ByDedi\Module\CustomForm\Service\FieldFromEntity;
use ByDedi\Module\CustomForm\Service\FormFromEntity;
use Form\Constraint\Callback;
use Form\Constraint\ConstraintInterface;
use Form\Constraint\EqualToField;
use Form\Constraint\Mail;
use Form\Constraint\NotBlank;
use Form\FullForm\FieldInterface;
use PHPUnit_Framework_TestCase;

class FormFromEntityTest extends PHPUnit_Framework_TestCase
{
    public function testGetFields()
    {
        $field = $this->getFieldEntity(
            'id1',
            FieldInterface::TYPE_TEXT_ID,
            'Le label',
            [
                $this->getConstraint(Mail::class, false),
                $this->getConstraint(EqualToField::class, true)
            ]
        );

        $field2 = $this->getFieldEntity(
            'id2',
            FieldInterface::TYPE_FILE_ID,
            'Fichier',
            [
                $this->getConstraint(Callback::class, false),
                $this->getConstraint(NotBlank::class, true)
            ]
        );

        $form = new FormFromEntity($this->getFormEntity('test1', 'success', [$field, $field2]));

        $this->assertEquals('test1', $form->getId());
        $this->assertEquals('success', $form->getSuccessText());
        $this->assertCount(2, $form->getFields());
    }

    /**
     * @throws \PHPUnit_Framework_AssertionFailedError
     */
    public function testWithError()
    {
        $form = new FormFromEntity($this->getFormEntity('test1', 'success'));
        $this->assertFalse($form->hasError());
        $this->assertNull($form->getErrorText());

        $formWithError = $form->withError('error message');

        $this->assertTrue($formWithError->hasError());
        $this->assertEquals('error message', $formWithError->getErrorText());

        $formWithoutError = $formWithError->withoutError();

        $this->assertFalse($formWithoutError->hasError());
        $this->assertNull($formWithoutError->getErrorText());
    }

    /**
     * @param $formId
     * @param $successText
     * @param array $fields
     * @return FormEntityInterface
     */
    public function getFormEntity($formId, $successText, array $fields = [])
    {
        $formEntity = $this
            ->getMockBuilder(FormEntityInterface::class)
            ->getMock()
        ;

        $formEntity
            ->method('getFields')
            ->willReturn($fields)
        ;
        $formEntity
            ->method('getSuccessText')
            ->willReturn($successText)
        ;
        $formEntity
            ->method('getId')
            ->willReturn($formId)
        ;

        return $formEntity;
    }

    /**
     * @param $fieldId
     * @param $typeId
     * @param string $label
     * @param array $constraints
     * @return FieldEntityInterface
     */
    public function getFieldEntity($fieldId, $typeId, $label = '', array $constraints = [])
    {
        $formEntity = $this
            ->getMockBuilder(FieldEntityInterface::class)
            ->getMock()
        ;

        $formEntity
            ->method('getConstraints')
            ->willReturn($constraints)
        ;
        $formEntity
            ->method('getOptions')
            ->willReturn([])
        ;
        $formEntity
            ->method('getOptionByValue')
            ->willReturn(null)
        ;
        $formEntity
            ->method('getTypeId')
            ->willReturn($typeId)
        ;
        $formEntity
            ->method('getId')
            ->willReturn($fieldId)
        ;
        $formEntity
            ->method('getLabel')
            ->willReturn($label)
        ;

        return $formEntity;
    }

    /**
     * @param $constraintClass
     * @param $isValid
     * @return ConstraintInterface
     */
    public function getConstraint($constraintClass, $isValid)
    {
        $constraint = $this
            ->getMockBuilder($constraintClass)
            ->disableOriginalConstructor()
            ->getMock()
        ;
        $constraint
            ->method('isValid')
            ->willReturn($isValid)
        ;

        return $constraint;
    }
}
