# Module Custom Form

Ce module permet de générer des formulaires entièrement ou partiellement editables en Back Office.
Ce module comprend de base:

* La possibilité de créer des champs de formulaire de type text, select, radio ou checkbox

* La possibilité d'ajouter des contraintes de validation a ces champs (non vide, mail ou telephone)

* L'enregistrement en BO des réponses au formulaire

Le Module n'a pas encore de version stable

# Installation

Ce module est compatible avec la v2.0 du ShopByDedi

Pour installer le module, assurez vous d'avoir le code suivant dans le composer.json
```json

  "require": {
    "oomphinc/composer-installers-extender": "^1.1"
  },
  "repositories": [
    {
      "type": "composer",
      "url": "https://packages.bydedi.com"
    }
  ],
  "extra": {
    "installer-types": ["bydedi-module"],
    "installer-paths": {
      "src/Dedi/Module/CustomForm": ["bydedi/custom-form"]
    }
  }

```

Vous pouvez ensuite installer le module ainsi

```shell
composer require bydedi/custom-form
```

Le module devrait être à présent dans le dossier src/Dedi/Module

Vous devez ensuite activer le module comme tous les autres dans config/modules.php par exemple ainsi

```php
<?php

return array(
    // Ligne a ajouter pour activer le module
    new ByDedi\Module\CustomForm\CustomForm(),
);

```

Pour les versions du ShopByDedi 2.0 antérieure au 09/07/18, assurez vous que votre phinx.yml comprend bien les modules comme ainsi
```yml
paths:
    migrations:
        - %%PHINX_CONFIG_DIR%%/db/migrations
        - %%PHINX_CONFIG_DIR%%/src/Dedi/Module/*/Resources/db
```

# Utilisation

## Services

Ces services vous permettent d'effectuer le comportement classique du module.

Si vous souhaitez modifier des comportements, les services peuvent simplement être hérité.
Il vous est cependant vivement recommandé dans ce cas là verouiller la version du module dans le composer.json

### ResponseFactory

Ce service permet de créer une FormResponse à partir d'une requête et d'un CustomForm, typiquement généré à partir d'un FormFactory
 
Exemple d'utilisation

```php
<?php

use ByDedi\Module\CustomForm\Model\FormEntity;
use ByDedi\Module\CustomForm\Service\FormFromEntity;

$form = new FormFromEntity(new FormEntity(1));

$this->template->assign('form', $form);
return $this->render('form.tpl');

```

```php
<?php

use ByDedi\Module\CustomForm\Service\ResponseFactory;
use ByDedi\Module\CustomForm\Model\FormEntity;
use ByDedi\Module\CustomForm\Service\FormFromEntity;
use ByDedi\Module\CustomForm\Service\FilledFormFromEntity;
use Symfony\Component\HttpFoundation\Request;

/** @var Request $request */
$form = new FormFromEntity(new FormEntity(1));
$filledForm = new FilledFormFromEntity($form, $request);

if (!$filledForm->isValid()) {
    $this->template->assign('form', $filledForm);
    return $this->render('form.tpl');
}

$responseFactory = new ResponseFactory();
$responseEntity = $responseFactory->generate($filledForm);
$responseEntity->save();
echo 'Votre formulaire a bien été enregistré';

```

## Model

Le module propose plusieurs Model pour récupérer les formulaires.
Dans la plupart des cas, vous ne devriez pas avoir à manipuler ces classes
 
## Views
 
Il faudrait proposer des templates de base pour les différents type de field.
A voir avec les front pour un style générique et applicable à tous les projets
 
## BO



# Développement

## Général

Si vous souhaitez faire une release du module, merci de lancer les tests auparavant pour valider le comportement du module.

Le plus simple pour développer le module est de se mettre sur la branche dev-module-custom-form du ShopByDedi, de faire un composer update, puis de remplacer src/Dedi/Module/CustomForm par un lien symbolique vers le dossier contenant le Module

## RoadMap

### FormFactory
- Finaliser les tests

### ResponseFactory
- A tester

### Components
- A tester

### Model
- A refactorise : notamment FormField::typeHasOption() dont le résultat devrait dépendre d'un champs de la table form_field_type
- A tester : FormResponse et FormResponseField

### Migrations

- A faire: détecter si le champs langue_id devrait être signed ou non.

### Views

- A faire: récupérer les views sur Ligne Roset, les adapter.
- A faire: documenter les différents blocks pour qu'un utilisateur puisse hériter de ces templates facilement.
 
### BO
 
- A faire: Ajouter les classes dans les migrations
- A tester

### Controllers

- A faire: Créer un controller d'exemple